/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
 *
 * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
 * Other names may be trademarks of their respective owners.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 *
 * Portions Copyrighted 2012 AgroSense
 */
package org.netbeans.modules.welcome.ui;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import javax.swing.Action;
import javax.swing.JPanel;
import org.netbeans.modules.welcome.content.ActionButton;
import org.netbeans.modules.welcome.content.Constants;
import org.netbeans.modules.welcome.content.Utils;
import org.openide.loaders.DataObject;
import org.openide.util.NbBundle;

/**
 *
 * @author S. Aubrecht
 */
@NbBundle.Messages({"ACD_GettingStarted=Open default web browser with documentation to get started with {0}"})
class GetStarted extends JPanel implements Constants {

    private static final String GETSTARTED_LINKS_ROOT_PATH = "WelcomePage/GettingStartedLinks";
    private int row;
    private ActionBuilder builder = new ActionBuilder();

    /**
     * Creates a new instance of RecentProjects
     */
    public GetStarted() {
        super(new GridBagLayout());
        setOpaque(false);
        buildContent();
    }

    private void buildContent() {
        String rootName = GETSTARTED_LINKS_ROOT_PATH;
        for (DataObject dataObject : builder.getDataObjects(rootName)) {
            row = addLink(row, dataObject);
        }
    }

    private int addLink(int row, DataObject dob) {
        Action action = builder.extractAction(dob);
        if (null != action) {
            JPanel panel = new JPanel(new GridBagLayout());
            panel.setOpaque(false);
            ActionButton lb = new ActionButton(action, Utils.getUrlString(dob),
                    COLOR_HEADER, true, dob.getPrimaryFile().getPath());
            panel.add(lb, new GridBagConstraints(1, 0, 1, 3, 1.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
            lb.setFont(GET_STARTED_FONT);

            lb.getAccessibleContext().setAccessibleName(lb.getText());
            lb.getAccessibleContext().setAccessibleDescription(Bundle.ACD_GettingStarted(lb.getText()));
            add(panel, new GridBagConstraints(0, row++, 1, 1, 1.0, 0.0,
                    GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                    new Insets(0, 0, 7, 0), 0, 0));
        }
        return row;
    }
}
